from django.test import TestCase
from django.test.utils import override_settings
from django.utils.functional import empty
from django.contrib.staticfiles.storage import staticfiles_storage


TEST_SETTINGS = {
    "AWS_STORAGE_BUCKET_NAME": "etianen-django-require-s3",
    "AWS_QUERYSTRING_AUTH": False,
    "CACHES": {
        "default": {
            "BACKEND": "django.core.cache.backends.dummy.DummyCache",
        },
    },
}


class ResetStaticFilesStorageMixin(object):

    def setUp(self):
        staticfiles_storage._wrapped = empty


@override_settings(
    STATICFILES_STORAGE = "require_s3.storage.OptimizedStaticFilesStorage",
    **TEST_SETTINGS
)
class TestOptimizedStaticFilesStorage(ResetStaticFilesStorageMixin, TestCase):

    def testFilePath(self):
        self.assertEqual(
            staticfiles_storage.url("test.png"),
            "https://etianen-django-require-s3.s3.amazonaws.com/test.png",
        )

    def testFolderPath(self):
        self.assertEqual(
            staticfiles_storage.url("test/"),
            "https://etianen-django-require-s3.s3.amazonaws.com/test/",
        )


@override_settings(
    STATICFILES_STORAGE = "require_s3.storage.OptimizedCachedStaticFilesStorage",
    **TEST_SETTINGS
)
class TestOptimizedCachedStaticFilesStorage(ResetStaticFilesStorageMixin, TestCase):

    def testFilePath(self):
        self.assertEqual(
            staticfiles_storage.url("test.png"),
            "https://etianen-django-require-s3.s3.amazonaws.com/test.5e3ae665fe7b.png",
        )

    def testFolderPath(self):
        self.assertEqual(
            staticfiles_storage.url("test/"),
            "https://etianen-django-require-s3.s3.amazonaws.com/test/",
        )



from setuptools import setup

from require_s3 import __version__


version_str = ".".join(str(n) for n in __version__)


setup(
    name = "django-require-s3",
    version = version_str,
    license = "BSD",
    description = "An integration between django-require and the S3 storage backend from django-storages.",
    author = "Dave Hall",
    author_email = "dave@etianen.com",
    url = "https://github.com/etianen/django-require-s3",
    packages = [
        "require_s3",
    ],
    zip_safe = False,
    install_requires = [
        "django-require",
        "django-storages",
        "boto",
    ],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Internet :: WWW/HTTP",
    ],
)



from django.contrib.staticfiles.storage import CachedFilesMixin

from storages.backends.s3boto import S3BotoStorage

from require.storage import OptimizedFilesMixin


class FolderNameHackMixin(object):

    # HACK: Fixing handling of folder names.
    def url(self, name, *args, **kwargs):
        url = super(FolderNameHackMixin, self).url(name, *args, **kwargs)
        if name.endswith("/") and not url.endswith("/"):
            url += "/"
        return url


class OptimizedStaticFilesStorage(FolderNameHackMixin, OptimizedFilesMixin, S3BotoStorage):
    
    """
    A storage backend that optimizes with RequireJS and stores the result on S3.
    """


class OptimizedCachedStaticFilesStorage(FolderNameHackMixin, OptimizedFilesMixin, CachedFilesMixin, S3BotoStorage):
    
    """
    A storage backend that optimizes with RequireJS, applies a cached MD5 suffix,
    and stores the result on S3.
    """
